Struct solana_program_test::ProgramTest [−][src]
pub struct ProgramTest { /* fields omitted */ }Implementations
impl ProgramTest[src]
impl ProgramTest[src]pub fn new(
program_name: &str,
program_id: Pubkey,
process_instruction: Option<ProcessInstructionWithContext>
) -> Self[src]
program_name: &str,
program_id: Pubkey,
process_instruction: Option<ProcessInstructionWithContext>
) -> Self
pub fn prefer_bpf(&mut self, prefer_bpf: bool)[src]
Override default BPF program selection
pub fn set_bpf_compute_max_units(&mut self, bpf_compute_max_units: u64)[src]
Override the BPF compute budget
pub fn add_account(&mut self, address: Pubkey, account: Account)[src]
Add an account to the test environment
pub fn add_account_with_file_data(
&mut self,
address: Pubkey,
lamports: u64,
owner: Pubkey,
filename: &str
)[src]
&mut self,
address: Pubkey,
lamports: u64,
owner: Pubkey,
filename: &str
)
Add an account to the test environment with the account data in the provided filename
pub fn add_account_with_base64_data(
&mut self,
address: Pubkey,
lamports: u64,
owner: Pubkey,
data_base64: &str
)[src]
&mut self,
address: Pubkey,
lamports: u64,
owner: Pubkey,
data_base64: &str
)
Add an account to the test environment with the account data in the provided as a base 64 string
pub fn add_program(
&mut self,
program_name: &str,
program_id: Pubkey,
process_instruction: Option<ProcessInstructionWithContext>
)[src]
&mut self,
program_name: &str,
program_id: Pubkey,
process_instruction: Option<ProcessInstructionWithContext>
)
Add a BPF program to the test environment.
program_name will also used to locate the BPF shared object in the current or fixtures
directory.
If process_instruction is provided, the natively built-program may be used instead of the
BPF shared object depending on the BPF_OUT_DIR environment variable.
pub async fn start(self) -> (BanksClient, Keypair, Hash)[src]
pub async fn start_with_context(self) -> ProgramTestContext[src]
Start the test client
Returns a BanksClient interface into the test environment as well as a payer Keypair
with SOL for sending transactions