Crate solana_program_test

Crate solana_program_test 

The solana-program-test provides a BanksClient-based test framework SBF programs

Re-exports

pub use tokio;

Macros

processor
Converts a solana-program-style entrypoint into the runtime’s entrypoint style, for use with ProgramTest::add_program

Structs

BanksClient
BanksTransactionResultWithMetadata
EbpfVm
A virtual machine to run eBPF programs.
EncryptedHostAddressToEbpfVm
Encrypted address to the EbpfVm object.
InvokeContext
Main pipeline from runtime to program execution.
MemoryMapping
Common parts of UnalignedMemoryMapping and AlignedMemoryMapping
ProgramTest
ProgramTestContext

Enums

BanksClientError
Errors from BanksClient
EbpfError
Error definitions
ProgramTestError
Errors from the program test environment

Traits

BuiltinFunctionDefinition
Native built-in functions that can be made available to programs to call.
ProgramTestBanksClientExt

Functions

find_file
get_runtime_environment_key
Returns (and if not done before generates) the encryption key for the VM pointer
invoke_builtin_function
read_file

Type Aliases

IndexOfAccount
Index of an account inside of the transaction or an instruction.